#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May 20 10:05:22 2020

@author: kevin
"""

import numpy as np                    
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
import pylab 
import sys
from scipy.signal import find_peaks, peak_prominences, peak_widths
from scipy.stats import sem

plt.rcParams.update({'font.size': 36})
plt.close()

# decide which plots you want to have                                                               
timeplt = False
useSem = False    # use standard error instead of standard deviation
plotPeaks = False # plot asteriks at identified peak locations
eventChar = False # plot explanation for event characteristics (only works for run 8715)

s = int(sys.argv[3])           # number of time steps
fill = 4                       # length of file number

# unit conversion
convert = True

inParmSub = int(sys.argv[2]) # input used to pick one parameter combination out of the ensemble
inPlusarf = int(sys.argv[6]) #
thr = float(sys.argv[7])     # ice volume threshold for peak identification
nBase = int(sys.argv[8])

# title
titletext = ''

# arrays containing different model setups
hyd25kmSmallRamp = np.linspace(9365,9369,5,dtype='int')
hyd12p5kmSmallRamp = np.linspace(9370,9374,5,dtype='int')
hyd6p25kmSmallRamp = np.linspace(9375,9379,5,dtype='int')
hyd0D3p125km = np.asarray([9276,9289,9302,9303,9304])

# pick and combine the same parameter combination out of every model setup
files = [hyd25kmSmallRamp[inParmSub],hyd12p5kmSmallRamp[inParmSub],hyd6p25kmSmallRamp[inParmSub],hyd0D3p125km[inParmSub]] # 3p125km small ramp as base

# paths and file names
file_name = 'idSQts.nn'
path = '/home/khank/glacdynH/res.dir/SQres.dir/nn.dir/'
pathStats = '/home/khank/glacdynH/wrk/Data/'
file_prefix = ''
path_out = path+'out.'+file_name+file_prefix+'.txt'#str(n_start).zfill(fill)+'_'+str(n_end).zfill(fill)+'.txt'

# variable to read in (0=t, 1=ct, 2=vol, 3=ar, 4=icdav, 5=icflux, 6=ua, 7=btpav)
var_str = ['t','ct','vol/e16', 'ar/e12', 'icdav', 'icflux/e9', 'ua', 'btpav']
variable = int(sys.argv[1])
print('chosen variable:', var_str[variable])

# figure
figSaveDes = sys.argv[5]
var_plt = ['t','ct','vol', 'ar', 'icdav', 'icflux', 'ua', 'btpav']
fig_name = file_name[:-3]+file_prefix+var_plt[variable]+figSaveDes+str(int(sys.argv[2]))+'.png'#str(n_start).zfill(fill)+'_'+str(n_end).zfill(fill)+'.pdf'
figsave = '/home/khank/glacdynH/wrk/Figs/'+fig_name
print ("read", file_name, "files")
print('#------------------------------------------------------------------#')


#-------------------------------------------------------------------------------#
# Reading files 
#-------------------------------------------------------------------------------#


# check if file is available
print('Start of  Program')
print('#------------------------------------------------------------------#')

file_number = []
fileNotAva = []
tt = 0

for i in range(len(files)):
    path_in = path+file_name+file_prefix+str(files[i]).zfill(fill)
    try:
        input = open(path_in,'r')
    except IOError:
        print("File",path_in,"not accessible")
        tt += 1
        fileNotAva.append(int(files[i]))
    file_number.append(int(files[i]))
if tt == 0:
    print('All files accessible')
else:
    print(tt, 'Files not accessible')

# create data matrix    
output = np.zeros((s+1,len(file_number)))
output[0,:] = file_number
print('read',len(file_number)-len(fileNotAva),'files')

# read files
fileCrashed = []
for file in range(len(file_number)):
    
    if file_number[file] in fileNotAva:
        print('Skip',file_number[file])
    else:
        path_in = path+file_name+file_prefix+str(file_number[file]).zfill(fill)
        input = open(path_in,'r')
        print('reading file',path_in)

        data = []
        for index in range(1):
            input.readline()
        for line in input:
            words = line.split()
            # check if end of file is reached
            if words[0] == '#':
                break
            # check if value exceeded given format
            if ('*' in words[variable]) == True:
                print('value exceeded given format')
                data.append(0.0)
            else: 
                try:     # cut out end of output file
                    data.append(float(words[variable]))
                except:
                    break

        if len(data)<s:
            print('run',file_number[file],'crashed')
            fileCrashed.append(int(file_number[file]))
        else:
            output[1:len(data)+1,file] = data


#-------------------------------------------------------------------------------#
# Saving output 
#-------------------------------------------------------------------------------#


#print('#------------------------------------------------------------------#')
#print('Saving File to', path_out)
#np.savetxt(path_out,output)


#-------------------------------------------------------------------------------#
# Reading data matrix 
#-------------------------------------------------------------------------------#


path_plot = path_out
input = output
file_number = input[0,:]
input = np.delete(input, 0, 0)

# unit conversion
if convert == True:
    # convert volume to cubic-kilometer
    if variable == 2:
        input = input*1e16/1e9
        var_str[variable] = 'Hudson Strait Ice Volume [km$^3$]'
    # convert ice flux to Sverdrup
    elif variable == 5:
        input = input*1e9/(60*60*24*365*1e6)
        var_str[variable] = 'Mid Hudson Strait Ice Flux [Sv]'
    elif variable == 7:
        var_str[variable] = "Average Hudson Strait \n"+r"Basal Temperature [$^\circ$C]"

#-------------------------------------------------------------------------------#
# Analyzing data matrix 
#-------------------------------------------------------------------------------#

time_start = 0
time_end = int(sys.argv[4])#100
time = np.linspace(time_start,time_end,len(input))

rmse = np.zeros(len(files))
meanBias = np.copy(rmse)
nC = np.copy(rmse)

#  https://acp.copernicus.org/articles/18/9741/2018/acp-18-9741-2018-t03.pdf
Nts = len(input[:,0])
baseSetup = input[:,nBase]         # which run is the base setup
baseMean = np.nanmean(baseSetup)
baseStd = np.nanstd(baseSetup)

print('#------------------------------------------------------------------#')
print('Calculate RMSE and mean bias')
for i in range(len(files)):
    if files[i] in fileNotAva:
        print('run',files[i],'not available, all characteristics set to NaN')
        rmse[i] = np.nan
        meanBias[i] = np.nan
        # this is also included in the nC value
        nC[i] = 1
        continue
    elif files[i] in fileCrashed:
        rmse[i] = np.nan
        meanBias[i] = np.nan
        print('run',files[i],'crashed, all characteristics set to NaN')
        nC[i] = 1
        continue
    elif np.count_nonzero(input[:,i])<1900 or np.count_nonzero(baseSetup)<1900:
        rmse[i] = np.nan
        meanBias[i] = np.nan
        print('Problem with File '+str(int(file_number[i]))+', non-zero entries='+str(int(np.count_nonzero(input[:,i])))+', all characteristics set to NaN')
        nC[i] = 1
        continue
    else:
        rmse[i] = (np.sqrt(np.sum((input[:,i] - baseSetup)**2)/Nts))/baseMean*100
        meanBias[i] = (np.sum(input[:,i] - baseSetup)/Nts)/baseMean*100

baseStr = 'hyd0D3p125km'
baseStr = 'Base: '+baseStr+':'

# comparison setups
testStr = ['hyd25kmSmallRamp','hyd12p5kmSmallRamp','hyd6p25kmSmallRamp','hyd0D3p125km'] # includes base setup as check

print('#------------------------------------------------------------------#')
print('Save file at',pathStats+'rmseMeanBias'+figSaveDes+'Parm'+str(inParmSub)+'.txt')
f = open(pathStats+'rmseMeanBias'+figSaveDes+'Parm'+str(inParmSub)+'.txt','w')
f.write('{:27s}   {:2s}   {:12s}   {:12s} \n'.format('Setup '.rjust(27),'nC'.rjust(2),'RMSE'.rjust(12),'mean bias'.rjust(12)))
for i in range(len(rmse)):
    testStr[i] = testStr[i]+':'
    f.write('{:27s}   {:2d}   {:12.3f}   {:12.3f} \n'.format(testStr[i].rjust(27),int(nC[i]),rmse[i],meanBias[i]))
f.close()

print('#------------------------------------------------------------------#')
print('End of Program')
print('#------------------------------------------------------------------#')
